[% WRAPPER 'layout.html' %]
		<div class="main">
			<h1>PlackX::Framework Example: [% page_name %]</h1>
			<p>
				Please write some text in the area below.
			</p>
			<form method="POST">
				<div>
					<textarea name="comments"></textarea>
				</div>
				<div>
					<input type="submit" value="Post">
				</div>
			</form>
		</div>
		<div class="footer">
			Lorem ipsum doler sit amet
		</div>
[% END %]
