{
  some_key => 'some_value',
  another_key => 'another_value',
  some_arrayref => [
    {
        first => { subkey => 'subvalue' },
    },
    {
        second => { another_subkey => 'another subvalue' },
    },
  ],
}
