<!DOCTYPE html>
<html>
	<head>
		<title>Template One!</title>
		<style type="text/css">
			body {
				font-family: Sans-Serif;
				margin: 2em;
			}
		</style>
	</head>
	<body>
		<h1>Hello from Template example.phtml!</h1>
		<p>The time() is [% PERL %] print time() [% END %].</p>
		<p>The query string is: "[% REQUEST.query_string %]".</p>
	</body>
</html>
