<!DOCTYPE html>
<html>
	<head>
		<title>404: File Not Found!</title>
		<style type="text/css">
			body {
				font-family: Sans-Serif;
				margin: 2em;
			}
		</style>
	</head>
	<body>
		<h1>404 Error!</h1>
		<p>
			You attempted to access a page named
			"[% REQUEST.route_param('template-name') %]"
			but it does not exist.
		</p>
	</body>
</html>
