<html>
	<head>
		<style type="text/css">
			body {
				font-family:  "Sans-serif", Helvetica, Arial;
				color:        #333;
				margin:       2em;
			}
			div.footer {
				border-top:   1px solid #333;
				font-size:    90%;
				font-style:   italic;
				padding:      0.5em 0;
			}
			div.main {
				min-height: 15em;
			}
			textarea {
				width: 32em;
				height: 8em;
			}
			input {
				padding: 0.5em 1.0em;
				margin:  1.0em 0;
				font-size: 1em;
				color: #555;
				font-weight: bold;
			}
		</style>
		<title>PlackX::Framework Example: [% page_name %]</title>
	</head>
	<body>[% content %]</body>
</html>
